use std::str::FromStr;

use crate::{config::ConfigWrapper, downloader::Downloader, manifest::{song::Song, Manifest}, util::is_supported_host};



pub async fn add(cfg: &ConfigWrapper, manifest: &mut Manifest, downloader: &mut Downloader, url: &Option<String>, name: &Option<String>, playlist: &Option<String>) -> anyhow::Result<()> {
    
    log::debug!("Playlist: {playlist:?}");
    log::debug!("url: {url:?}");
    log::debug!("name: {name:?}");

    let mut playlists = manifest.get_playlists().keys().map(|f| f.clone()).collect::<Vec<String>>();

    playlists.sort();

    let playlist = playlist.clone().unwrap_or_else( || {
        let g = crate::prompt::prompt_with_list_or_str("Enter song playlist", &playlists);
        log::info!("Playlist: {g}");
        g
    });


    let url = url.clone().unwrap_or_else( ||
        crate::prompt::simple_prompt("Enter song youtube url, make sure its not a playlist, (yt only for now)")
    );

    if !is_supported_host(url::Url::from_str(&url)?) {
        log::error!("Invalid or unsupported host name");
        return Ok(());
    }


    let name = name.clone().unwrap_or_else( ||
        crate::prompt::simple_prompt("Enter song name with like this: {Author} - {Song name}")
    );

    let song = Song::from_url_str(url)?;
    manifest.add_song(playlist.clone(), name.clone(), song.clone());
    manifest.save(None)?;

    let should_download = crate::prompt::prompt_bool("Download song now?", Some(false));

    if should_download {
        downloader.download_song(cfg, &name, &song, &playlist, manifest.get_format())?;
        crate::process_manager::wait_for_procs_untill(0)?;
    }

    Ok(())
}
